from obt._dep_build import BaseBuilder
from obt._dep_impl import require
from obt import pathtools, path, _globals
from obt.command import Command
import obt.host
from collections.abc import Callable

###############################################################################
# BinInstaller : install binaries from downloaded package
###############################################################################

class BinInstaller(BaseBuilder):
  """Install binaries from downloaded package"""
  ###########################
  class InstallerItem:
    def __init__(self,src,dst,flags):
      self._src = src
      self._dst = dst
      self._flags = flags
  ###########################
  def __init__(self,name):
    super().__init__(name)
    self._items = []
    self._OK = True
  ###########################################
  """declare an install item given a source-path and dest-path"""
  def install_item(self,
                   source=None,
                   destination=None,
                   flags=None):
    self._items += [BinInstaller.InstallerItem(source,destination,flags)]
  ###########################################
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    ok2build = require(self._deps)
    if not ok2build:
      return False
    for item in self._items:
      exists = item._src.exists()
      #print(item,item._src,exists)
      if False==exists:
        self._OK = False
        return False
    return True
  ###########################################
  def install(self,blddir):
    for item in self._items:
      cmd = [
        "cp", item._src, item._dst
      ]
      Command(cmd).exec()
      if item._flags != None:
        cmd = [
          "chmod",item._flags, item._dst
        ]
        Command(cmd).exec()
    return self._OK




#!/usr/bin/env python3

from obt import command, host

def _getLinkedLibrariesLinux(executable_path, unique_set, libraries):
  result = command.capture(["ldd", executable_path])
  for line in result.strip().split("\n"):
    parts = [part.strip() for part in line.split("=>")]
    lib_name = parts[0].split(" ")[0]
    if len(parts) > 1:
      lib_path = parts[1].split(" ")[0]
    else:
      lib_path = None  # Some system libraries might not have a path
    libraries[lib_name] = lib_path
    if lib_path and lib_path not in unique_set:
      unique_set.add(lib_path)
      _getLinkedLibrariesLinux(lib_path, unique_set,libraries)

def getLinkedLibraries(executable_path):
  unique_set = set()
  libraries = {}
  if host.IsLinux:
    _getLinkedLibrariesLinux(executable_path,unique_set,libraries)
    return libraries
  else:
    assert(False)
    return None



from obt import command
class Session(object):
  #########################################
  def __init__(self, session_name, orientation="horizontal",working_dir=None):
    self.session_name = session_name
    self.orientation = orientation
    self.working_dir = working_dir

    self.cmd_new_session = [
      "tmux", "new-session",
      "-d", "-s", f"{self.session_name}"]
    if self.orientation == "vertical":
      self.cmd_add_session = [
        "tmux", "split-window",
        "-v", "-t", f"{self.session_name}"]
      self.cmd_select_layout = [
        "tmux", "select-layout",
        "-t", f"{self.session_name}:0",
        "even-vertical"
      ]
    elif self.orientation == "horizontal":
      self.cmd_add_session = [
        "tmux", "split-window",
        "-h", "-t", f"{self.session_name}"]
      self.cmd_select_layout = [
        "tmux", "select-layout",
        "-t", f"{self.session_name}:0",
        "even-horizontal"
      ]
    self.cmd_attach_session = [
      "tmux", "attach-session",
      "-t", f"{self.session_name}"
    ]
    self.kill_session = [
      "tmux", "kill-session",
      "-t", f"{self.session_name}"
    ]
    if working_dir!=None:
      self.cmd_new_session.append("-c")
      self.cmd_new_session.append(working_dir)
      self.cmd_add_session.append("-c")
      self.cmd_add_session.append(working_dir)
    
    self.cmd_chain = command.chain2(do_log=True)
    self.post_chain = list()
  #########################################
  def bind_key(self, key=None, cmd=None, table="prefix" ):
    """Bind a key to a command in the tmux session."""
    assert( key is not None and cmd is not None )
    self.post_chain.append([
      "tmux", "bind-key",
      "-T", table,
      key, cmd,
      "-t", f"{self.session_name}",
    ])
  #########################################
  def kill(self):
    try:
      command.run(kill_session, check=False)
    except:
      pass
  #########################################
  def first_command(self, cmd):
    self.cmd_chain.add(self.cmd_new_session + cmd)
  #########################################
  def next_command(self, cmd):
    self.cmd_chain.add(self.cmd_add_session + cmd)
  #########################################
  def command(self, cmd):
    if self.cmd_chain.count == 0:
      self.first_command(cmd)
    else:
      self.next_command(cmd)
  #########################################
  def select_layout(self):
    self.cmd_chain.add(self.cmd_select_layout)
  #########################################
  def attach_session(self):
    self.cmd_chain.add(self.cmd_attach_session)
  #########################################
  def execute(self):
    for item in self.post_chain:
      self.cmd_chain.add(item)
    self.select_layout()
    self.attach_session()
    OK = (self.cmd_chain.execute() == 0)
    return OK


###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, path
###############################################################################
class tracy(dep.StdProvider):
  name = "tracy"
  def __init__(self):
    super().__init__(tracy.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    #self._builder.setCmVar("CMAKE_INSTALL_INCLUDEDIR",path.includes()/"tracy")
    #self._builder.requires(["eigen"])

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=tracy.name,
                             repospec="wolfpld/tracy",
                             revision="v0.9.1",
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"readme.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libtracy.so").exists()
  



#!/usr/bin/env python3 

from yarl import URL
from ork import command, path, pathtools, wget


builddir = path.builds()/"amigadev-test1"
pathtools.chdir(builddir)

archive = URL("https://archive.org/download")

# fetch kickstarts
zipname = "Amiga-Kickstart-ROMs-Commodore-A500-A600-A1000-A3000-A4000-A1200-A2000.zip"
url = archive/"amiga-kickstart-roms-commodore-a-500-a-600-a-1000-a-3000-a-4000-a-1200-a-2000"/zipname
wget.wget( urls=[url],
           output_name=path.downloads()/zipname,
           md5val="2765730b20470e7001d656d1f49b721d")
command.run(["unzip","-o",path.downloads()/zipname])

# fetch DOS disk
zipname = "AmigaDOS CLI rev 27.5 (1985)(Commodore)[m].zip"
wget.wget( urls=[archive/"CommodoreAmigaApplicationsADF"/zipname],
           output_name=path.downloads()/zipname,
           md5val="f5b9e49849dc04ddb35b1d6de3155fa2")
command.run(["unzip","-o",path.downloads()/zipname])

command.run(["fs-uae",
             "--hard_drive_0=.",
             "--floppy-drive-0=%s"%str(builddir/"img.adf"),
             "--amiga-model=a4000 ",
             "--kickstart_file=./Kickstart-v3.1-rev40.70-1994-Commodore-A4000.rom",
             "--floppy_drive_volume=0"])



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, string

###############################################################################

class Theme:
  def __init__(self,bash):
    super().__init__()
    self.bash = bash
  ###############################
  def rgb256(self,r,g,b):
    r = int((r*5)/255)
    g = int((g*5)/255)
    b = int((b*5)/255)
    color = 16 + 36 * r + 6 * g + b
    rval = "\033[38;5;%dm" % color
    if self.bash:
      rval = "\\[" + rval + "\\]"
    return rval
  ###############################
  def blink(self):
    return "\033[5m"
  ###############################
  def vrgb256(self,r,g,b):
    return self.rgb256(r,g,b)
  ###############################
  def rgbstr(self,r,g,b,string):
    return self.rgb256(r,g,b)+str(string)+self.reset()
  ###############################
  def reset(self):
    rval = "\033[m"
    if self.bash:
      rval = "\\["+rval+"\\]"
    return rval
  ###############################
  def key(self,string):
    return self.vrgb256(255,255,0)+str(string)+self.reset()
  def val(self,string):
    return self.vrgb256(255,255,255)+str(string)+self.reset()
  def path(self,string):
    return self.vrgb256(255,255,128)+str(string)+self.reset()
  def inf(self,string):
    return self.vrgb256(128,128,255)+str(string)+self.reset()
  def warn(self,string):
    return self.yellow(string)+self.reset()
  def err(self,string):
    return self.red(string)+self.reset()
  ###############################
  def bright(self,string):
    return self.vrgb256(255,255,255)+str(string)+self.reset()
  ###############################
  def magenta(self,string):
    return self.vrgb256(255,0,255)+str(string)+self.reset()
  def cyan(self,string):
    return self.vrgb256(0,255,255)+str(string)+self.reset()
  def white(self,string):
    return self.vrgb256(255,255,255)+str(string)+self.reset()
  def orange(self,string,blink=False):
    bl = self.blink() if blink else ""
    return self.vrgb256(255,128,0)+bl+str(string)+self.reset()
  def yellow(self,string):
    return self.vrgb256(255,255,0)+str(string)+self.reset()
  def red(self,string):
    return self.vrgb256(255,0,0)+str(string)+self.reset()
  ###############################
  def promptL(self,string):
    return self.vrgb256(255,0,0)+str(string)+self.reset()
  ###############################
  def promptC(self,string):
    return self.vrgb256(255,255,0)+str(string)+self.reset()
  ###############################
  def promptR(self,string):
    return self.vrgb256(255,128,0)+str(string)+self.reset()
  ###############################

###############################################################################

class DarkTheme(Theme):
  def __init__(self,bash=False):
    super().__init__(bash=bash)
  ###############################
  def val(self,string):
    return self.vrgb256(255,192,192)+str(string)+self.reset()
  ###############################
  def bright(self,string):
    return self.vrgb256(255,255,128)+str(string)+self.reset()

###############################################################################

class InverseTheme(Theme):
  def __init__(self,bash=False):
    super().__init__(bash=bash)
  ###############################
  def vrgb256(self,r,g,b):
    return self.rgb256(255-r,255-g,255-b)
  ###############################
  def promptL(self,string):
    return self.rgb256(255,0,0)+str(string)+self.reset()
  ###############################
  def promptC(self,string):
    return self.rgb256(255,255,0)+str(string)+self.reset()
  ###############################
  def promptR(self,string):
    return self.rgb256(255,128,0)+str(string)+self.reset()
  #def rgbstr(self,r,g,b,string):
  #  return super().rgb256(255-r,255-g,255-b)+str(string)+self.reset()

###############################################################################

class LightTheme(Theme):
  def __init__(self,bash=False):
    super().__init__(bash=bash)
  ###############################
  def key(self,string):
    return self.rgb256(25,25,0)+str(string)+self.reset()
  def val(self,string):
    return self.rgb256(0,160,160)+str(string)+self.reset()
  def path(self,string):
    return self.rgb256(65,65,65)+str(string)+self.reset()
  def inf(self,string):
    return self.rgb256(15,15,30)+str(string)+self.reset()
  def warn(self,string):
    return self.yellow(string)+self.reset()
  def err(self,string):
    return self.red(string)+self.reset()
  ###############################
  def bright(self,string):
    return self.rgb256(0,0,0)+str(string)+self.reset()
  ###############################
  def vrgb256(self,r,g,b):
    return self.rgb256(255-r,255-g,255-b)
  ###############################
  def promptL(self,string):
    return self.rgb256(128,0,0)+str(string)+self.reset()
  ###############################
  def promptC(self,string):
    return self.rgb256(128,128,0)+str(string)+self.reset()
  ###############################
  def promptR(self,string):
    return self.rgb256(128,96,0)+str(string)+self.reset()

###############################################################################

class MonoTheme(Theme):
  def __init__(self,bash=False):
    super().__init__(bash=bash)
  ###############################
  def rgb256(self,r,g,b):
    return ""
  ###############################
  def reset(self):
    return ""

###############################################################################

class Deco:
  
  ###############################
  def __init__(self,bash=False):
    if "OBT_THEME" in os.environ:
      if os.environ["OBT_THEME"]=="dark":
        self._theme = DarkTheme(bash=bash)
      elif os.environ["OBT_THEME"]=="light":
        self._theme = LightTheme(bash=bash)
      elif os.environ["OBT_THEME"]=="inverse":
        self._theme = InverseTheme(bash=bash)
      elif os.environ["OBT_THEME"]=="mono":
        self._theme = MonoTheme(bash=bash)
    else:
      self._theme = DarkTheme(bash=bash)
  ###############################
  def rgbstr(self,r,g,b,string):
    return self._theme.rgbstr(r,g,b,string)
  ###############################
  def magenta(self,string):
    return self._theme.magenta(string)
  def cyan(self,string):
    return self._theme.cyan(string)
  def white(self,string):
    return self._theme.white(string)
  def orange(self,string,blink=False):
    return self._theme.orange(string,blink=blink)
  def yellow(self,string):
    return self._theme.yellow(string)
  def red(self,string):
    return self._theme.red(string)
  ###############################
  def key(self,string):
    return self._theme.key(string)
  def val(self,string):
    return self._theme.val(string)
  def path(self,string):
    return self._theme.path(string)
  def inf(self,string):
    return self._theme.inf(string)
  def warn(self,string):
    return self._theme.warn(string)
  def err(self,string):
    return self._theme.err(string)
  ###############################
  def bright(self,string):
    return self._theme.bright(string)
  ###############################
  def promptL(self,string):
    return self._theme.promptL(string)
  ###############################
  def promptC(self,string):
    return self._theme.promptC(string)
  ###############################
  def promptR(self,string):
    return self._theme.promptR(string)

###############################################################################

class DecoFormatter(string.Formatter):
  def __init__(self):
    self.deco = Deco()    
  def format_field(self, value, format_spec):
    if isinstance(value, str):
      if format_spec.endswith('red'):
        value = self.deco.red(value)
        format_spec = format_spec[:-3]
    return super(DecoFormatter, self).format(value, format_spec)

###############################################################################

__all__ =	[ "deco" ]



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION ="toz-obt"

import os, tarfile
from obt import dep, host, path, git, cmake, make
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command
from obt.cmake import context

deco = Deco()

###############################################################################

class easyprof(dep.StdProvider):
  name = "easyprof"
  #############################################
  def __init__(self):
    super().__init__(easyprof.name)
    self.declareDep("cmake")
    qt5 = dep.instance("qt5")
    mpaths = [
      qt5.basedir/"lib"/"cmake"
    ]

    self._builder = self.createBuilder( dep.CMakeBuilder,
                                        modules_paths=mpaths)

    self._builder.setCmVar("CMAKE_PREFIX_PATH",qt5.basedir/"lib"/"cmake"/"Qt5Widgets")
  ########################################################################
    # todo post install on mac
    #  copy path.libs()/"libeasy_profiler.dylib" to dep(python).library_dir
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=easyprof.name,
                             repospec="tweakoz/easy_profiler",
                             revision=VERSION,
                             recursive=False)

  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libeasy_profiler.so").exists()



#!/usr/bin/env python3

import os, sys, pathlib, argparse
from obt import dep, host, path

if len(sys.argv)==2:
	dep.require(sys.argv[1])
else:
    print( "usage: dep.require.py <depname>")
    sys.exit(1)
